// -*- C++ -*-
/*!
 * @file  ManipulatorControlSample.cpp
 * @brief ManipulatorControlSample
 * @date $Date$
 *
 * $Id$
 */

#include "ManipulatorControlSample.h"
#include <math.h>

// Module specification
// <rtc-template block="module_spec">
static const char* manipulatorcontrolsample_spec[] =
  {
    "implementation_id", "ManipulatorControlSample",
    "type_name",         "ManipulatorControlSample",
    "description",       "ManipulatorControlSample",
    "version",           "1.0.0",
    "vendor",            "rsdlab",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Speed", "20",

    // Widget
    "conf.__widget__.Speed", "text",
    // Constraints

    "conf.__type__.Speed", "int",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ManipulatorControlSample::ManipulatorControlSample(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
	m_inIn("in", m_in),
    m_ManipulatorCommonInterface_CommonPort("ManipulatorCommonInterface_Common"),
    m_ManipulatorCommonInterface_MiddlePort("ManipulatorCommonInterface_Middle")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
ManipulatorControlSample::~ManipulatorControlSample()
{
}



RTC::ReturnCode_t ManipulatorControlSample::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
	addInPort("in", m_inIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_ManipulatorCommonInterface_CommonPort.registerConsumer("ManipulatorCommonInterface_Common", "JARA_ARM::ManipulatorCommonInterface_Common", m_ManipulatorCommonInterface_Common);
  m_ManipulatorCommonInterface_MiddlePort.registerConsumer("ManipulatorCommonInterface_Middle", "JARA_ARM::ManipulatorCommonInterface_Middle", m_ManipulatorCommonInterface_Middle);
  
  // Set CORBA Service Ports
  addPort(m_ManipulatorCommonInterface_CommonPort);
  addPort(m_ManipulatorCommonInterface_MiddlePort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Speed", m_Speed, "20");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ManipulatorControlSample::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControlSample::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControlSample::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ManipulatorControlSample::onActivated(RTC::UniqueId ec_id)
{
  //プロバイダより遅くActivateするため
  sleep(1);

  JARA_ARM::JointPos jointPoint;
  jointPoint.length(4);
  jointPoint[0] = 0;
  jointPoint[1] = 65;
  jointPoint[2] = -35;
  jointPoint[3] = 55;

  m_rid = m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jointPoint);

  //関節速度の設定
  JARA_ARM::ULONG spdRation;
  spdRation = m_Speed;//%
  m_ManipulatorCommonInterface_Middle->setSpeedJoint(spdRation);

  /*リミット設定
  //ジョイントソフトリミットの初期化(AX-12A対応)
  JARA_ARM::LimitSeq softLimit;
  softLimit.length(5);
  softLimit[0].upper = 300;
  softLimit[0].lower = 0;
  softLimit[1].upper = 300;
  softLimit[1].lower = 0;
  softLimit[2].upper = 300;
  softLimit[2].lower = 0;
  softLimit[3].upper = 300;
  softLimit[3].lower = 0;
  m_ManipulatorCommonInterface_Common->setSoftLimitJoint(softLimit);

  //範囲ソフトリミットの初期化(CRANE+対応)
  JARA_ARM::LimitValue xLimit;
  JARA_ARM::LimitValue yLimit;
  JARA_ARM::LimitValue zLimit;
  xLimit.upper = 329;
  xLimit.lower = -300;
  yLimit.upper = 314.5;
  yLimit.lower = -314.5;
  zLimit.upper = 359.9;
  zLimit.lower = 0;
  m_ManipulatorCommonInterface_Middle->setSoftLimitCartesian(xLimit, yLimit, zLimit);
  */

  return RTC::RTC_OK;
}


RTC::ReturnCode_t ManipulatorControlSample::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ManipulatorControlSample::onExecute(RTC::UniqueId ec_id)
{
  int c;
  
  if (m_inIn.isNew())
  {

	  m_inIn.read();
	  c = m_in.data;
	  printf("%d",c);

	  if (c == 0)
	  {
		  m_rid = m_ManipulatorCommonInterface_Middle->moveGripper(50);
		  std::cout << "close" << std::endl;
		  if (m_rid->id != 0) {//Error
			  std::cout << "moveGripper ERROR" << std::endl;
			  std::cout << m_rid->comment << std::endl << std::endl;
		  }
	  }

	  else if (c == 1)
	  {
		  m_rid = m_ManipulatorCommonInterface_Middle->openGripper();
		  std::cout << "open" << std::endl;
		  if (m_rid->id != 0) {//Error
			  std::cout << "openGripper ERROR" << std::endl;
			  std::cout << m_rid->comment << std::endl << std::endl;
		  }
	  }

	  else
		  std::cout << "無効な値です" << std::endl << std::endl;
  }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ManipulatorControlSample::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControlSample::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControlSample::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControlSample::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControlSample::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ManipulatorControlSampleInit(RTC::Manager* manager)
  {
    coil::Properties profile(manipulatorcontrolsample_spec);
    manager->registerFactory(profile,
                             RTC::Create<ManipulatorControlSample>,
                             RTC::Delete<ManipulatorControlSample>);
  }
  
};


